@@ -15,14 +15,10 @@ server configuration and auth.
 import http.client
 import json
 import logging
-import time
-import uuid
+import sys
+import traceback
 from typing import List
 
-import google.auth
-import google.auth.transport.requests
-from google.oauth2 import service_account
-from jose import jwt
 from pydantic import BaseModel
 
 from metadata.config.common import ConfigModel
@@ -34,6 +30,7 @@ from metadata.generated.schema.entity.data.topic import Topic
 from metadata.generated.schema.entity.services.databaseService import DatabaseService
 from metadata.generated.schema.entity.tags.tagCategory import Tag
 from metadata.ingestion.ometa.auth_provider import AuthenticationProvider
+from metadata.ingestion.ometa.client import APIError
 
 logger = logging.getLogger(__name__)
 
@@ -81,6 +78,7 @@ class MetadataServerConfig(ConfigModel):
     email: str = None
     audience: str = "https://www.googleapis.com/oauth2/v4/token"
     auth_header: str = "X-Catalog-Source"
+    scopes: List = []
 
 
 class NoOpAuthenticationProvider(AuthenticationProvider):
@@ -91,8 +89,11 @@ class NoOpAuthenticationProvider(AuthenticationProvider):
     def create(cls, config: MetadataServerConfig):
         return cls(config)
 
-    def auth_token(self) -> str:
-        return "no_token"
+    def auth_token(self):
+        pass
+
+    def get_access_token(self):
+        return ("no_token", None)
 
 
 class GoogleAuthenticationProvider(AuthenticationProvider):
@@ -104,12 +105,21 @@ class GoogleAuthenticationProvider(AuthenticationProvider):
         return cls(config)
 
     def auth_token(self) -> str:
+        import google.auth
+        import google.auth.transport.requests
+        from google.oauth2 import service_account
+
         credentials = service_account.IDTokenCredentials.from_service_account_file(
             self.config.secret_key, target_audience=self.config.audience
         )
         request = google.auth.transport.requests.Request()
         credentials.refresh(request)
-        return credentials.token
+        self.generated_auth_token = credentials.token
+        self.expiry = credentials.expiry
+
+    def get_access_token(self):
+        self.auth_token()
+        return (self.generated_auth_token, self.expiry)
 
 
 class OktaAuthenticationProvider(AuthenticationProvider):
@@ -124,24 +134,90 @@ class OktaAuthenticationProvider(AuthenticationProvider):
     def create(cls, config: MetadataServerConfig):
         return cls(config)
 
-    def auth_token(self) -> str:
-        from okta.jwt import JWT  # pylint: disable=import-outside-toplevel
+    async def auth_token(self) -> str:
+        import time
+        import uuid
+        from urllib.parse import quote, urlencode
 
-        _, my_jwk = JWT.get_PEM_JWK(self.config.private_key)
-        claims = {
-            "sub": self.config.client_id,
-            "iat": time.time(),
-            "exp": time.time() + JWT.ONE_HOUR,
-            "iss": self.config.client_id,
-            "aud": self.config.org_url + JWT.OAUTH_ENDPOINT,
-            "jti": uuid.uuid4(),
-            "email": self.config.email,
-        }
-        token = jwt.encode(claims, my_jwk.to_dict(), JWT.HASH_ALGORITHM)
-        return token
+        from okta.cache.okta_cache import OktaCache
+        from okta.jwt import JWT, jwt
+        from okta.request_executor import RequestExecutor
+
+        try:
+            my_pem, my_jwk = JWT.get_PEM_JWK(self.config.private_key)
+            issued_time = int(time.time())
+            expiry_time = issued_time + JWT.ONE_HOUR
+            generated_jwt_id = str(uuid.uuid4())
+            claims = {
+                "sub": self.config.client_id,
+                "iat": issued_time,
+                "exp": expiry_time,
+                "iss": self.config.client_id,
+                "aud": self.config.org_url,
+                "jti": generated_jwt_id,
+            }
+            token = jwt.encode(claims, my_jwk.to_dict(), JWT.HASH_ALGORITHM)
+            config = {
+                "client": {
+                    "orgUrl": self.config.org_url,
+                    "authorizationMode": "BEARER",
+                    "rateLimit": {},
+                    "privateKey": self.config.private_key,
+                    "clientId": self.config.client_id,
+                    "token": token,
+                    "scopes": self.config.scopes,
+                }
+            }
+            request_exec = RequestExecutor(
+                config=config, cache=OktaCache(ttl=expiry_time, tti=issued_time)
+            )
+            parameters = {
+                "grant_type": "client_credentials",
+                "scope": " ".join(config["client"]["scopes"]),
+                "client_assertion_type": "urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
+                "client_assertion": token,
+            }
+            encoded_parameters = urlencode(parameters, quote_via=quote)
+            url = f"{self.config.org_url}?" + encoded_parameters
+            token_request_object = await request_exec.create_request(
+                "POST",
+                url,
+                None,
+                {
+                    "Accept": "application/json",
+                    "Content-Type": "application/x-www-form-urlencoded",
+                },
+                oauth=True,
+            )
+            _, res_details, res_json, err = await request_exec.fire_request(
+                token_request_object[0]
+            )
+            if err:
+                raise APIError(f"{err}")
+            response_dict = json.loads(res_json)
+            self.generated_auth_token = response_dict.get("access_token")
+            self.expiry = response_dict.get("expires_in")
+        except Exception as err:
+            logger.debug(traceback.print_exc())
+            logger.error(err)
+            sys.exit()
+
+    def get_access_token(self):
+        import asyncio
+
+        asyncio.run(self.auth_token())
+        return (self.generated_auth_token, self.expiry)
 
 
 class Auth0AuthenticationProvider(AuthenticationProvider):
+    """
+    OAuth authentication implementation
+    Args:
+        config (MetadataServerConfig):
+    Attributes:
+        config (MetadataServerConfig)
+    """
+
     def __init__(self, config: MetadataServerConfig):
         self.config = config
 
@@ -160,4 +236,9 @@ class Auth0AuthenticationProvider(AuthenticationProvider):
         res = conn.getresponse()
         data = res.read()
         token = json.loads(data.decode("utf-8"))
-        return token["access_token"]
+        self.generated_auth_token = token["access_token"]
+        self.expiry = token["expires_in"]
+
+    def get_access_token(self):
+        self.auth_token()
+        return (self.generated_auth_token, self.expiry)
